function [A,t,r] = TransformMatrixMDDT(z,lambda,delta0,deltaZ,N,M,av)
% TransformMatrixMDDT() calculates the averaged or non-averaged M-DDT [1] 
% transform matrix used for the precise forward wave field propagation of 
% the object wave field to the parallel transverse plane at the distance z 
% in the spatial domain. These transfrom matrices are construction by assuming
% averaging within pixels of the object and sensor plane wave fields to obtain
% accurate pixel-to-pixel propagation model. The Fresnel approximation of the 
% Rayleigh-Sommerfeld integral is used as a diffraction kernel.
%
% Notation:
% [A,t,r] = TransformMatrixMDDT(z,lambda,delta0,deltaZ,N,M,av)
% ------------------------------- input -----------------------------------
% z                     propagation distance between the object and sensor planes [m]
% lambda                wavelength [m]
% delta0                pitch size of the discrete object wave field w.r.t. rows (columns) [m]
% deltaZ                pitch size of the discrete sensor plane wave field w.r.t. rows (columns) [m] 
% N                     number of rows (columns) of the object distribution
% M                     number of rows (columns) of the sensor plane distribution
% av                    switch on/off (av = 1/0) the averaging for the M-DDT 
%                       transform matrices (optional, by default av = 1). 
%                       If av = 0, the non-averaged transform matrices result 
%                       in the conventional Fresnel discrete diffraction transfrom.
% Note! If delta0==deltaZ the transform matrix is calculated using a single 1D data array. If in this
% case av>1 (say, av=2), the direct element-wise calculated is performed as in case delta0~=deltaZ.
% ------------------------------- output ----------------------------------
% A                     averaged or non-averaged M-DDT transfrom matrix 
%                       of the size M x N (depending on the input parameter
%                       av, see Eq. (17) and Eq. (24) in [1]).
% t                     the computational time in seconds (optional)
% r                     the numerical rank of the transform matrix (optional)
%
%
% Example:
% Calculation of the sensor plane wave field uz of the size M1 x M2 using the 
% amplitude object u0 of the size N1 x N2 with different rectangular pixels at the
% object and sensor planes of the size delta0y x delta0x and deltaZy x deltaZx, respectively.
%
% u0 = im2double(imread('Lena256.png'));                 % read the test-image
% [N1,N2,N3] = size(u0);                                 % image size
% lambda = 6328e-10;                                     % wavelength (red laser)
% delta0y = 6e-6; delta0x = 8e-6;                        % rectangular pixel size for the object
% deltaZy = delta0y; deltaZx = delta0x;                  % and for the sensor plane wave fields
% M1 =  N1; M2 = N2;                                     % the sensor plane wave field is taken of the same size as the object size
% z = delta0y*deltaZy*M1/lambda;                         % propagation distance
% Ay = TransformMatrixMDDT(z,lambda,delta0y,deltaZy,N1,M1);
% Ax = TransformMatrixMDDT(z,lambda,delta0x,deltaZx,N2,M2);
% uz = Ay*u0*(Ax.');                                     % the complex-valued sensor plane wave field
%
% Reference:            [1] V. Katkovnik, A. Migukin, and J. Astola, Appl. Opt. 48, 3407-3423 (2009). 
% Authors:              Artem Migukin, Vladimir Katkovnik
% Software version:     2.0, January 2013.

opt = true;             % computational optimization by a single computing of the matrix elements with 
                        % the same coordinate values (use of the evenness of the integral).
                        % Set "opt=false" for the direct element-wise calulation of the
                        % transform matrix (depending on the size it can be very slow).
                        
tol = 10e-12;           % tolarance used to calculate the numerical rank of the M-DDT transform matrix          

          
%% --------------- please do not change the code under this line -----------------------------------------
if (nargin<7 || isempty(av)),    av=true;      end                          % by default the averaging for
                                                                            % transform matrix is switched on 
if (N<1 || M<1) || (ceil(N)-floor(N)==1 || ceil(M)-floor(M)==1)
    disp('Error: the size of the transform matrix must be a positive integer')
    A = [];t=0;r=[]; return
end
if abs(z)<lambda
    disp('Error: too small propagation distance, "z" must be larger "lambda"!') 
    A = [];t=0;r=[]; return 
end
i = sqrt(-1);
if nargout > 1,    tic;end                                                  % start a stopwatch timer 

mufactor = exp(i*pi*z/lambda)/sqrt(i*z*lambda);                         % mu factor in [1, Eq. (24)] 
[X,Y] = meshgrid((-N/2:N/2-1)*delta0,(-M/2:M/2-1)*deltaZ);                  % sampling grid
t = i*pi/z/lambda;                                                          % common exponent
a=(Y-X);a2=a.^2;

if av == 0                                                                      % non-averaged matrix
    A = mufactor*exp(t*(a2))*delta0; 
else
    A = zeros(M,N);                                                         % preallocation of the matrix
    
    if delta0==deltaZ && av == 1                                            % the simplest case, the same pixels  
                                                                            % at the object and sensor planes
        disp('calculation with a shortcut algorithm')
        sq = -N/2-M/2+1:N/2+M/2;                                            % sampling grid
        p = ones(1,length(sq));                                             % preallocation
        % calculation of the matrix using shifting
        
        for q=1:length(sq)                                                  % calculation of 1D data array
            p(q)=quad(@(x)kernel(x,t,sq(q),delta0),-1,1); 
            % p(q) = exp( t*delta0^2*(sq(q).^2) ); < to proof in case of non-averaging 
        end        
        for qy = 1:M                                                        % filling of the transform matrix by shifting
            A(qy,:) = p(N+qy-1:-1:qy);
        end
    else                                                                    % different pixel sizes
        D1 = (deltaZ+delta0)/2; D2 = abs(delta0-deltaZ)/2;                  % limits of integral [1, Eq. (51)]
        
        if opt  % calculation of double integral according to Appendix 2 in [1]
             b = ones(M,N);                                                 % auxilary indices        
             disp('optimization: use the evenness of the integral formula...') 
             for index = 1:N*M             
                % do not consider elements which are equal to previous ones
                if b(index)~=0
                % indexes of equal elements in absolute value of the matrix 
                    tt = find(ismember(a2,a2(index)));            b(tt)=0;         
                    % calculation of the double integral as a single one
                    value=...
                        1/max([deltaZ delta0])*quad(@(x)J1opt(x,t,a(index)),-D2,D2)+...        % J1
                        1/deltaZ/delta0*quad(@(x)J2opt(x,t,a(index),deltaZ,delta0),D2,D1)+...  % J2
                        1/deltaZ/delta0*quad(@(x)J3opt(x,t,a(index),deltaZ,delta0),D2,D1);     % J3                
                    A(tt)=value; % all elements with equal coordinates should have the same value
                end
             end         
        else % direct calculation of each element of the matrix 
            disp('direct calculation with a standard algorithm...') 
            for n=-N/2:N/2-1
                 for m=-M/2:M/2-1
                    A(m+M/2+1,n+N/2+1)=...
                        1/max([deltaZ delta0])*quad(@(x)J1(x,t,m,n,deltaZ,delta0),-D2,D2)+... % J1
                        1/deltaZ/delta0*quad(@(x)J2(x,t,m,n,deltaZ,delta0),D2,D1)+...         % J2
                        1/deltaZ/delta0*quad(@(x)J3(x,t,m,n,deltaZ,delta0),D2,D1);            % J3
                 end
            end
        end        
    end
    A = mufactor*A*delta0;
end

if nargout > 1
    t=toc; % stop the stopwatch timer - obtain the time of calculation
end
if nargout > 2
    r = rank(A'*A,tol); % numerical rank
end

% inline function for numerical integration of the single integral
function y = kernel(x,t,d,delta)
    y = (1-abs(x)).*exp(t*delta^2*(d+x).^2);

    
% inline functions for numerical integration of the single integral (52)
function y=J1opt(x,t,ks) 
    y = exp(t*(ks+x).^2);
function y=J2opt(x,t,ks,deltaZ,delta0)
    y = exp(t*(ks+x).^2).*((delta0+deltaZ)/2-x);
function y=J3opt(x,t,ks,deltaZ,delta0)
    y = exp(t*(ks-x).^2).*((delta0+deltaZ)/2-x);


function y=J1(x,t,k,s,deltaZ,delta0) 
    y = exp(t*(k*deltaZ-s*delta0+x).^2);
function y=J2(x,t,k,s,deltaZ,delta0)
    y = exp(t*(k*deltaZ-s*delta0+x).^2).*((delta0+deltaZ)/2-x);
function y=J3(x,t,k,s,deltaZ,delta0)
    y = exp(t*(k*deltaZ-s*delta0-x).^2).*((delta0+deltaZ)/2-x);    
% 2008 - 2013 (c) TUT v. 2.0