%% Demo for the complex-valued sensor plane wave field calculation and the object reconstruction
% Here we calculate the complex-valued wave field at the sensor plane and reconstruct the object by these complex-valued observations.
% Since there is no noise this demo serves to represent the invertibility of the data obtained by M-DDT [1] using the Tikhonov regularized inverse
% of M-DDT, by the truncated SVD, the F-DDT recursive regularized inverse [2] and by the conventional convolutional and angular spectrum decomposition methods.
% The simulation is presented for the amplitude- and phase-only objects, where the signal is constructed by a test-image.
% The parameters of the optical system can be set manually (please set manual=1) or using a GUI (be default, manual=0).
% 
% ------------------------------- input -----------------------------------
% u0                    the object wave field (amplitude- or phase-only, formed via the chosen test-image)
% imagetype             the type of the object, imagetype = 1 (or  = 0) means the amplitude- (or phase-) only object
% delta0y               pixel size of the discrete object wave field w.r.t. rows [m]
% delta0x               pixel size of the discrete object wave field w.r.t. columns [m]
% deltaZy               pitch size of the discrete sensor plane wave field w.r.t. rows [m] 
% deltaZx               pitch size of the discrete sensor plane wave field w.r.t. columns [m]
% lambda                wavelength [m]
% q = [qy,qx]           redundancy of the sensor (the ratio of the sesor plane wave field distribution 
%                       w.r.t. the object one) in the y- and x-direction, respectively (by default for the same 
%                       size of the object and sensor plane wave fields qy = 1, qx = 1, must be qy >= 1, qx >= 1).
% d                     propagation distance between the object and sensor planes [m] is set via the factor for the so-called "in-focus" distance dF,
%                       calculated as dF = min(qy*Ny0*delta_y_z*delta_y_0/lambda, qx*Nx0*delta_x_z*delta_x_0/lambda), where Ny0 x Nx0 is the object size.
%                       For example, the perfect reconstruction is quaranteed for d = 1*dF, i.e. at the "in-focus" distance. For larger or smaller distance of the forward
%                       and backward diffraction propagation use the following factors:
% fff                   the factor to set the forward diffraction propagation: fff=1 means d = fff*dF
% bbb                   the factor to set the backward diffraction propagation: bbb=1 means d = bbb*dF
%                       
% Note! the system is very sensitive to the accurate distance and if fff not equal to bbb, one may be in trouble :) Use e.g. the regularization parameters ...
% alpha                 the regularization parameter for M-DDT, in the GUI it is named by "reg. MDDT" for better distinguishing with no comments
% beta                  the regularization parameter for F-DDT RRI, in the GUI it is named by "reg. FDDT" to distinguish it from the regularization parameter for M-DDT
% smoothing_f           averaging for the forward propagation for M-DDT
% smoothing_b           averaging for the backward propagation for M-DDT 
%                       smoothing_f/b = 0 (or smoothing_f/b = 1) means non-averaged (or averaged) matrices used for the forward/backward propagation
% smoothing_back        averaging for the backward propagation for F-DDT 
% illustrations         the M-DDT object reconstruction can be compared by different other methods, namely:
%                       with truncated SVD, the truncated SVD, the F-DDT recursive regularized inverse [2] and by the conventional convolutional and angular spectrum decomposition methods
%                       Use the control parameter "illustrations" which determine which of these corresponding method to use for comparison.
% ------------------------------- output ----------------------------------
% Imaging of the reconstructed object (its amplitude and phase) by all these methods, and reconstruction accuracy in RMSE
% You can save the parameters and the M-DDT transform matrices if you set the additional parameter "SaveMatrices" equal to 1 (true).
% Also you can load the saved data: for this purpose set the parameter "LoadMatrices" equal to 1 (true).
% 
% Reference:            [1] V. Katkovnik, A.Migukin, J. Astola, Appl. Opt. 48, 3407-3423 (2009).
%                       [2] V. Katkovnik, J. Astola, and K. Egiazarian, Appl. Opt. 47, 3481-3493 (2008).
% Authors:              Artem Migukin, Vladimir Katkovnik
% Software version:     2.0, January 2013.
% Required functions:   TransformMatrixMDDT(), TransferFunctionFDDT(), TransferFunctionASD(), MDDT(), FDDT()


manual = 0; % if manual = 1 - we use manual settings , otherwise (manual = 0) - a user interface 
% initialization
if manual % manual settings
    clc;    clear;    close all ; manual=1;
    
    % ========================== user parameters ==========================
        
    imagetype = 1;                          % the type of a test image: 1/0 - amplitude/phase only
    % the pitch sizes (different in the general case)
    delta_x_0 = 8e-6; delta_y_0 = 5e-6;     % the object pitch sizes  
    delta_x_z = 8e-6; delta_y_z = 5e-6;     % the sensor pitch sizes
    lambda = 632.8e-9;                      % wavelength
    qy=1.0; qx=1.0;                         % the size ratio between the object and sensor sizes q = Nz/N0
    fff = 1;                                % the distance for the forward propagation for manual settings
    bbb = 1;                                % a set of distances for the backward propagation / reconstructions in multiple manner, 
                                            % e.g. bbb = 1 means the same backward propagation distance as the forward one, 
                                            % bbb = 0.5 means a half of the forward propagation distance        
    alpha = 1e-12;                          % regularization parameter for inverse matrices 
    beta = 0.01;                            % regularizing parameter for the FDDT method
    smoothing_f = 1;                        % averaging for the forward propagation for M-DDT
    smoothing_b = 1;                        % averaging for the backward propagation for M-DDT 
                                            % = 0 (or = 1) means non-averaged (or averaged) matrices used for the forward/backward propagation
    smoothing_back = 1;                     % averaging for for the backward F-DDT propagation
    illustrations = [1 0 0 0 0 0];        % illustration of the reconstruction by the truncated SVD, F-DDT RRI, conv1, conv2, ASD1 and ASD2, respectively.
    
    
    
    
    SaveMatrices = 0;% set this parameter to 1 in order to save transfer matrices
    LoadMatrices = 0;% set this parameter to 1 in order to use saved transfer matrices
    
% --------------- please do not change the code under this line -----------------------------------------  
    uishuttdown = 1; % shutting down the user dialog for image choosing and use the most simple image (if =1) 
    kappay = 1;kappax = 1;  % scaling factors (in this version they arent used, DO NOT CHANGE!!!)
 SoTI = dir('*.png');% a set of test images
 if size(SoTI,1)>=1
    for b=1:size(SoTI,1)
            imagesizes(b) = SoTI(b).bytes;
    end
    [v vindex] = max(imagesizes); 
 else
        disp('in the current directory there is no .png file')
 end
 if (size(SoTI,1)>=1) && ~uishuttdown 
        u0 = imread(SoTI(vindex).name); 
 else
     % a set of test images
    SoTI = {
    'cameraman128.png'    
    'goldhill.png'
    'Lena256.png'
    'baboon512.png'
    'chessboard128.jpg'
    'logo256.bmp'
    };
    while 1 
% choice of the test image (with an interactive menu)
    Km = menu('Choose a test image','cameraman (128x128)','goldhill (256x256)',...
        'Lena (256x256)','baboon (512x512)','chessboard (128x128)',...
        'logo (256x256)','image browser','exit');
% as the object u0 we take an only-amplitude object A(x,y)*exp(-j*phi(x,y)),
% where phi(x,y) = 0 for all x,y and A(x,y) is equal to the intensity of
% the current test image
        if Km == 9     % read the name of the file from the command window        
         % we take one image from the standard user dialog
        [filename, pathname] = uigetfile( ...
{  '*.BMP;*.CUR;*.GIF;*.HDF4;*.ICO;*.JPEG;*.PBM;*.PCX;*.PGM;*.PNG;*.PPM;*.RAS;*.TIFF;*.XWD','Image files';
   '*.BMP','BMP image (*.BMP)';...
   '*.CUR','Cursor image (*.CUR)'; ...
   '*.GIF','GIF image (*.GIF)'; ...
   '*.HDF4','HDF4 image (*.HDF4)'; ...
   '*.ICO','Icon image (*.ICO)'; ...
   '*.JPEG','JPEG image (*.JPEG)'; ...
   '*.PBM','PBM image (*.PBM)'; ...
   '*.PCX','PCX image (*.PCX)'; ...
   '*.PGM','PGM image (*.PGM)'; ...
   '*.PNG','PNG image (*.PNG)'; ...
   '*.PPM','PPM image (*.PPM)'; ...
   '*.RAS','SUN raster image (*.RAS)'; ...
   '*.TIFF','TIFF image (*.TIFF)'; ...
   '*.XWD','XWD image (*.XWD)'}, ...
   'Choose an image file for processing');
            if filename~=0% we are waiting for choosing an image file
                u0 = imread([pathname filename]);            break
            end
        elseif Km>0 && Km<9
        % we check for file existing
        filename = char(SoTI(Km));
            if exist(filename,'file')
            % we take one image from the offered set        
                u0 = imread(filename);        break
            else
                msgbox('Take another file or use "image browser" button','There isnt such a file!!!');
            end
        else
            break
        end    
    end
    close all
        if Km == 10 || Km == 0    
            break
        end
 end
 
 if SaveMatrices
     if 1 % set 1 to use a dialog in order to set where you want to save the data <<<<<<<<<<<<<<<<
       [filename, pathname] = uiputfile( ...
               {'*.mat','MAT-files (*.mat)'}, ...        
               'Save transfer matrices as', 'data.mat');       
       if ~ischar(filename)
            uiwait(msgbox('No data will be saved!','Warning!!!','modal'));            
            datafilename = [];
       else
           datafilename = [pathname filename];
           uiwait(msgbox(['Transfer matrices will be saved in: ' datafilename],'Note!','modal'));           
       end
     else % save to the last *.mat file
         filelist_mat_structure = dir('*.mat');         
        for ii=1:length(filelist_mat_structure)
            pp(ii)=filelist_mat_structure(ii).datenum;
        end
        [v index]=max(pp);filename=filelist_mat_structure(index).name;
        p = mfilename('fullpath');pathname=p(1:max(find(p=='\')));
        datafilename = [pathname filename];
     end
 end
 
 if LoadMatrices && ~SaveMatrices 
     ButtonName1Yes=0;
   outdata = [lambda qy qx delta_y_0 delta_x_0 delta_y_z delta_x_z kappay kappax alpha beta];
   if 0 % use the last saved *.mat file <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        filelist_mat_structure = dir('*.mat');         
        for ii=1:length(filelist_mat_structure)
            pp(ii)=filelist_mat_structure(ii).datenum;
        end
        [v index]=max(pp);filename=filelist_mat_structure(index).name;
        p = mfilename('fullpath');pathname=p(1:max(find(p=='\')));
        datafilename = [pathname filename];
   else  % set 1 to use a dialog in order to set which *.mat file you want to load <<<<<<<<<<<<
     [filename, pathname] = uigetfile( ...
     {  '*.mat','MAT-files(*.mat)';},...
     'Choose a MAT-file');        
    if ~ischar(filename)
        datafilename = []; 
        uiwait(msgbox('no *.mat file was chosen!','Warinig!!!','modal'));              
    else 
     while 1
            datafilename = [pathname filename];
            p=whos('-file', datafilename);x=cell(1,length(p));
            for ii=1:length(p)    
                x{ii}=p(ii).name;
            end
%             yes_no=all([strcmp(x{6},'delta_y_0') strcmp(x{4},'delta_x_0') strcmp(x{7},'delta_y_z') ...
%             strcmp(x{5},'delta_x_z') strcmp(x{10},'lambda') strcmp(x{12},'qy') strcmp(x{11},'qx')...
%             strcmp(x{9},'kappay') strcmp(x{8},'kappax') strcmp(x{3},'alpha') strcmp(x{2},'Ay') strcmp(x{1},'Ax')]);       
            yes_no=all([strcmp(x{7},'delta_y_0') strcmp(x{5},'delta_x_0') strcmp(x{8},'delta_y_z') ...
                strcmp(x{6},'delta_x_z') strcmp(x{11},'lambda') strcmp(x{13},'qy') strcmp(x{12},'qx')...
                strcmp(x{10},'kappay') strcmp(x{9},'kappax') strcmp(x{3},'alpha') strcmp(x{2},'Ay') strcmp(x{1},'Ax') strcmp(x{4},'beta')]);


            p1=load(datafilename);   
            if size(u0,3)>1 % in this demo we consider gray scale images only
                u0 = rgb2gray(u0);
            end 
            [Ny0_,Nx0_] = size(u0);   Ny0 = Ny0_ + mod(Ny0_,2);    Nx0 = Nx0_ + mod(Nx0_,2);            
         if  ~(size(p1.Ax,2)  == Nx0  &&  size(p1.Ay,2)  == Ny0)
                     [filename, pathname] = uigetfile( ...
                     {  '*.mat','MAT-files(*.mat)';},...
                     'The transfer matrices in the chosen file are not correspond with the chosen image!Choose another MAT-file');
                 if ~ischar(filename)
                        uiwait(msgbox('No *.mat file was chosen!','Warinig!!!','modal'));                     
                        datafilename = [];
                        break
                  end
         else   
            if yes_no            
             
                %y{1}=x{10};y{2}=x{12};y{3}=x{11};y{4}=x{6};y{5}=x{4};y{6}=x{6};y{7}=x{5};y{8}=x{9};y{9}=x{8};y{10}=x{3};                
                y{1}=x{10+1};y{2}=x{12+1};y{3}=x{11+1};y{4}=x{6+1};y{5}=x{4+1};y{6}=x{6+1};y{7}=x{5+1};y{8}=x{9+1};y{9}=x{8+1};y{10}=x{3};y{11}=x{4};
                
                d(1) = p1.lambda;d(2) = p1.qy;d(3) = p1.qx; d(4) = p1.delta_y_0;d(5) = p1.delta_x_0;
                d(6) = p1.delta_y_z; d(7) = p1.delta_x_z;d(8) = p1.kappay;d(9) = p1.kappax; d(10) = p1.alpha;  d(11) = p1.beta;      
                correct_data=cell(1,10+1);kk=0;
                for jj=1:10+1
                    if ~(d(jj)==outdata(jj))
                        kk=kk+1;
                        correct_data{kk}=[y{jj} ': ' num2str(outdata(jj)) ' / ' num2str(d(jj))];
                    end
                end   
                
                if ~isempty(correct_data{1})                     
    %if we have at least 1 difference between current data and data from the chosen file, we need to control that situation
                    while 1
                        ButtonName = questdlg('Some differences between current settings and data from the chosen file were detected', ...
                             'Do you want to use the chosen *.mat file?', ...
                             'Yes, I will use the chosen file', 'No, I will take another one', 'Show me the differences','Yes, I will use the chosen file'); 
                    
                        if strcmp(ButtonName,'Show me the differences')
                            warning off
                            uiwait(Warning(datafilename,outdata));% show this differences                         
                            warning on 
                            ButtonName1 = questdlg('Do you want to save the current settings?', ...
                             'Save the current settings data?', ...
                             'Yes', 'No', 'Yes'); 
                            if  strcmp(ButtonName1,'Yes')     
                               [filename, pathname] = uiputfile( ...
                               {'*.mat','MAT-files (*.mat)'}, ...        
                               'Save transfer matrices as', 'data.mat');       
                               if ~ischar(filename)
                                    uiwait(msgbox('No data will be saved!','Warning!!!','modal'));            
                                    datafilename = [];
                               else
                                   datafilename = [pathname filename];
                                   uiwait(msgbox(['Transfer matrices will be saved in: ' datafilename],'Note!','modal'));           
                               end
                               LoadMatrices =0; SaveMatrices = 1;ButtonName1Yes=1;
                               break                            
                            end
                        else
                            break
                        end       
                    end 
                        if (strcmp(ButtonName,'Yes, I will use the chosen file')  ||  ButtonName1Yes) && ischar(filename)  
                            
                           
                            p1=load(datafilename);
                            lambda = p1.lambda;qy = p1.qy;qx = p1.qx; delta_y_0 = p1.delta_y_0;delta_x_0 = p1.delta_x_0;
                delta_y_z = p1.delta_y_z;delta_x_z = p1.delta_x_z;kappay = p1.kappay;kappay = p1.kappay; alpha = p1.alpha;  beta = p1.beta;
                            
                            break
                        elseif strcmp(ButtonName,'No, I will take another one')
                            [filename, pathname] = uigetfile( ...
                            {  '*.mat','MAT-files(*.mat)';},...
                            'Choose a MAT-file');  
                            if ~ischar(filename)                                
                                uiwait(msgbox('no *.mat file was chosen!','Warinig!!!','modal'));                                 
                                datafilename = [];break
                            end                                            
                        elseif strcmp(ButtonName,'')
                            uiwait(msgbox('no *.mat file was chosen!','Warinig!!!','modal'));                                
                            datafilename = [];break
                        end 
     
                else
                    break
                end  
            else
              
                [filename, pathname] = uigetfile( ...
                {  '*.mat','MAT-files(*.mat)';},...
                'Format of the *.mat file is not correct!Choose a MAT-file'); 
                  if ~ischar(filename)
                        uiwait(msgbox('no *.mat file was chosen!','Warinig!!!','modal'));                     
                        datafilename = [];
                        break
                  end
            end
         end
     end
    end
   end
 end 
 pushbutton = 1; % just a service variable (it allows performing the further algorithm)
else %% GUI menu    
    if nargin ~= 3
        clc
        clear
        close all
        % GUI processing and parameter settings 
        gui_menu 
        pushbutton = 0;
    else 
        imagetype = a; % 1/0 = AM/PM objects
        lambda = outputdata(1);qy = outputdata(2);qx = outputdata(3);
        kappay = outputdata(8);kappax = outputdata(9);
        alpha = outputdata(10);
        beta = outputdata(11); % ADD <<<<<<<<<<<<<<<<<<
        delta_y_0= outputdata(4);delta_x_0= outputdata(5);delta_y_z= outputdata(6);delta_x_z= outputdata(7);        
        smoothing_f = outputdata(11+1);% method for the forward propagation for M-DDT
        smoothing_b = outputdata(12+1);% method for the backward propagation for M-DDT
        illustrations = outputdata((13+1):end); % + ASD-1, ASD-2 at the end
        datafilename = address2save_load{1};
        
        
        
        if address2save_load{2}==1 
            SaveMatrices=1;LoadMatrices=1;            
        elseif address2save_load{2}==2  
            SaveMatrices=0;LoadMatrices=1;            
        elseif address2save_load{2}==0 
            SaveMatrices=0;LoadMatrices=0;        
        end
    end    
end

if pushbutton  
     i = sqrt(-1);
    if size(u0,3)>1 % in this demo we consider gray scale images only
        u0 = rgb2gray(u0);
    end 
    [Ny0_,Nx0_] = size(u0);
    % evenness control: if not we use zero padding to enlarge the image  
    Ny0 = Ny0_ + mod(Ny0_,2);    Nx0 = Nx0_ + mod(Nx0_,2);
    % correction of the image size in order to make it even    
    u = zeros(Ny0,Nx0);u(1:Ny0_,1:Nx0_)=u0;
    % converting the class of the variable from int8 to double
    u0 = im2double(uint8(u));
    q = [qy qx];
    %% Common settings
    clear qr pp
    SpeedCntr = 1;% in order to calculate the speed of all algorithms set it to 1
    % we consider the 'in-focus' distance in general case for backward propagation
    dFy = qy*Ny0_*delta_y_z*delta_y_0/lambda;dFx = qx*Nx0_*delta_x_z*delta_x_0/lambda;    
    % a correction of the 'in-focus' distance because of non-evenness of a test image     
    if manual
% for the averaged matrices the distance is min of 'in-focus' distances (see Fig. 10 in [1])       
        if smoothing_f && smoothing_b
            dF=min([dFx dFy]);
        else
            dF=max([dFx dFy]);%in general case we take the maxima 
        end
        
        setd = fff*dF;                      % the distance for the forward propagation for manual settings
        bsetd = setd*bbb;                   % a set of distances for the backward propagation / reconstructions 
    else
        if smoothing_f && smoothing_b
            setd=d(1)/dF*min([dFx dFy]);
            bsetd=d(2)/dF*min([dFx dFy]);
        else
            setd=d(1)/dF*max([dFx dFy]);
            bsetd=d(2)/dF*max([dFx dFy]);
        end
    end
    % compute a set of distances for which some results of numerical simulation were presented in [1]     
    if 0 % construct a set of distances if it is necessary
        a0=linspace(min([dFx dFy])/2,min([dFx dFy]),5);
        a1=linspace(min([dFx dFy]),mean([dFx dFy]),5);
        a2=linspace(mean([dFx dFy]),max([dFx dFy]),5);
        a3=linspace(max([dFx dFy]),3*max([dFx dFy])/2,5);
        setd = [a0(1:end-1) a1(1:end-1) a2(1:end-1) a3(1:end)];
    end
    %% F-DDT settings
    if illustrations(2)  
        if ~manual            
            smoothing_back = smoothing_b;% 0/1 == non-averaged/averaged matrices are used for the backward F-DDT propagation:
        end
    end    
 %% =================== Demo algorithm ====================================    
    if ~imagetype % construct a phase object if it necessary (if =1)
        u0 = ones(size(u0)).*exp(i*pi*(u0-1/2));
    end
    %% Calculation of the wavefield in the sensor plane
    % preallocation for speed
    rankAx = zeros(1,length(setd));rankAy = zeros(1,length(setd));
    condAx = zeros(1,length(setd));condAy = zeros(1,length(setd));
    RMSE_MDDT= zeros(1,length(setd));RMSE_MDDTa= zeros(1,length(setd));
    RMSE_opt_r= zeros(1,length(setd));RMSE_opt_ra= zeros(1,length(setd));
    RMSE_FreqS= zeros(1,length(setd));RMSE_FreqSa= zeros(1,length(setd));
    RMSE_FreqL= zeros(1,length(setd));RMSE_FreqLa= zeros(1,length(setd));
    RMSE_iDFrT= zeros(1,length(setd));RMSE_iDFrTa= zeros(1,length(setd));
    % ASD-1,2
    RMSE_ASD1= zeros(1,length(setd));RMSE_ASD2= zeros(1,length(setd));
%     u_norm2=sum(abs(u0(:)).^2);
    counter = 1; % counter for the next loop 
    h = waitbar(0,'Calculating in progress, please wait...'); % wait bar to inform about the processing
    if ~SaveMatrices && LoadMatrices && ~isempty(datafilename) 
        tic;data=load(datafilename);p1_=toc;
    end
 figure(2);figure(3);close(2);close(3)   

  for distance = setd(1:end)
    d = distance;
    disp(['distance (forward) = ' num2str(d) ' m ' '(' num2str(d/dF) '*dF)'])
    % calculation of the wavefield in the output plane with M-DDT method
    clear uz Ay Ax
    % calculation of a common coefficient mu for the concrete distance d
    %Mxy=exp(i*pi*(d/lambda+3/4))/sqrt(d*lambda);kx_ = Mxy*delta_x_0;ky_ = Mxy*delta_y_0;
    %disp(['version 1.2: the regularization parameter differs from the previous one (version 1.1) by |mu_x| * |my_y| = ' num2str(abs(kx_)*abs(ky_)) ' times'])
    %disp('alpha v.1.2 * |mu_x| * |my_y| = alpha v.1.1')
    if ~SaveMatrices && LoadMatrices && ~isempty(datafilename)         
        if data.xisaveraged       % use saved matrices       
            Ay=data.Ay;Ax=data.Ax;
            disp('M-DDT: use computed data for the wavefield calculation')
            %[uz,Ay,Ax,p_,rankAy(counter),rankAx(counter)] = function_model_MDDT(u0,[],d,lambda,q,delta_y_0,delta_x_0,delta_y_z,delta_x_z,smoothing_f,0,[],Ay,Ax);
            [uz,p_,Ay,Ax,rankAy(counter),rankAx(counter)] = MDDT(u0,0,d,lambda,delta_y_0,delta_x_0,delta_y_z,delta_x_z,q,smoothing_f,[],Ay,Ax);
        else
            disp('M-DDT: non-averaged matrices were saved in the chosen file.')
            %[uz,Ay,Ax,p_,rankAy(counter),rankAx(counter)] = function_model_MDDT(u0,[],d,lambda,q,delta_y_0,delta_x_0,delta_y_z,delta_x_z,smoothing_f,0,[]);
            [uz,p_,Ay,Ax,rankAy(counter),rankAx(counter)] = MDDT(u0,0,d,lambda,delta_y_0,delta_x_0,delta_y_z,delta_x_z,q,smoothing_f,[]);
        end
        p_=p_+p1_; 
    else
        %[uz,Ay,Ax,p_,rankAy(counter),rankAx(counter)] = function_model_MDDT(u0,[],d,lambda,q,delta_y_0,delta_x_0,delta_y_z,delta_x_z,smoothing_f,0,[]);
        [uz,p_,Ay,Ax,rankAy(counter),rankAx(counter)] = MDDT(u0,0,d,lambda,delta_y_0,delta_x_0,delta_y_z,delta_x_z,q,smoothing_f,[]);
        if SaveMatrices && ~isempty(datafilename) % save matrices for the current scenario 
            if ~smoothing_f
                uiwait(msgbox('Non-averaged matrices will be saved in the file. The flag "xisaveraged" will be equal to 0','Warinig!!!','modal'));             
            end 
            xisaveraged = smoothing_f;
            save(datafilename, 'delta_y_0', 'delta_x_0', 'delta_y_z', 'delta_x_z', 'lambda', 'qy', 'qx', 'kappay', 'kappax', 'alpha', 'beta', 'Ay', 'Ax', 'xisaveraged');
        end
    end
    
%     % calculation of the numerical condition of matrices 
    condAx(counter) = cond(transpose(Ax'*Ax));
    condAy(counter) = cond(Ay'*Ay);
    
    [Nyz,Nxz]=size(uz); % size of the wavefield in the sensor plane
    disp(['output wavefield size is ' num2str(Nyz) 'x' num2str(Nxz)])
    figure(1);
    subplot(1,2,1);imshow(abs(uz),[]);title('sensor plane amplitude')
    subplot(1,2,2);imshow(angle(uz),[]);title('sensor plane phase')
    
    %% Reconstruction of the wavefield at the object plane    
    for d = bsetd
        disp(['distance (backward) = ' num2str(d) ' m ' '(' num2str(d/distance) '*d)'])
        clear u0est y_oF u0FrS u0FrL u0FrS_ u0FreqL u0DFrT u0DFrT_ u0ASD_1 u0ASD_2
        %% wavefield reconstruction with the M-DDT method 
        waitbar((counter/length(setd)/length(bsetd))/3,h);
        if d==distance && smoothing_b
            disp('M-DDT: use computed data for the wave field reconstruction')            
            %[u0est,Ay,Ax,p] = function_model_MDDT(u0,uz,d,lambda,q,delta_y_0,delta_x_0,delta_y_z,delta_x_z,smoothing_b,1,alpha,Ay,Ax);
            [u0est,p,Ay,Ax] = MDDT(uz,1,d,lambda,delta_y_0,delta_x_0,delta_y_z,delta_x_z,q,smoothing_b,alpha,Ay,Ax);
        else
            %[u0est,Ay,Ax,p] = function_model_MDDT(u0,uz,d,lambda,q,delta_y_0,delta_x_0,delta_y_z,delta_x_z,smoothing_b,1,alpha);
            [u0est,p,Ay,Ax] = MDDT(uz,1,d,lambda,delta_y_0,delta_x_0,delta_y_z,delta_x_z,q,smoothing_b,alpha);
        end
        if SpeedCntr
            disp(['time cost for the M-DDT algorithm = ' num2str(p+p_) ' seconds'])        
        end

        % estimation of the accuracy of the wavefield reconstruction via RMSE 
        RMSE_MDDT(counter) = sqrt(mean(mean((abs(u0(1:Ny0_,1:Nx0_)) - abs(u0est(1:Ny0_,1:Nx0_))).^2))); % numerical comparison for the intensity
        
        xxx = angle(u0est(1:Ny0_,1:Nx0_)) - angle(u0(1:Ny0_,1:Nx0_));   mMDDT = mean(xxx(:));     
        RMSE_MDDTa(counter) = sqrt(mean(mean(( xxx-mMDDT ).^2)));% numerical comparison for the phase
        qr=1;pp=2;    waitbar((counter/length(setd)/length(bsetd))*2/3,h);
        
        
      if 1 %% comparing with other methods
        if illustrations(1) %% the inverse discrete Fresnel transform (M-IDFrT) %% <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
            if SpeedCntr
                tic
            end  
                        
            %Byd = function_Matrix_MDDT(-d,lambda,delta_y_0,delta_y_z,Ny0,Nyz,0)/conj(ky_);
            %Bxd = function_Matrix_MDDT(-d,lambda,delta_x_0,delta_x_z,Nx0,Nxz,0)/conj(kx_);
            %u0DFrT=(Byd*uz*transpose(Bxd))*conj(Mxy)*conj(Mxy)*delta_x_z*delta_y_z; % M-IDFrT                      
            u0DFrT = MDDT(uz,2,d,lambda,delta_y_0,delta_x_0,delta_y_z,delta_x_z,q,smoothing_b); 
            
            if SpeedCntr
                p=toc;disp(['time cost for the truncated SVD = ' num2str(p) ' seconds'])                
            end
            
            pp=3; % <- add new position (if this method is selected)
            RMSE_iDFrT(counter) = sqrt(mean(mean((abs(u0(1:Ny0_,1:Nx0_)) - abs(u0DFrT(1:Ny0_,1:Nx0_))).^2)));
            
            xxx = angle(u0DFrT(1:Ny0_,1:Nx0_)) - angle(u0(1:Ny0_,1:Nx0_)); mMiDFrT = mean(xxx(:));           
            RMSE_iDFrTa(counter) = sqrt(mean(mean(( xxx-mMiDFrT ).^2)));
        end
        if illustrations(2)  %% FDDT RRI         
            if SpeedCntr
                tic
            end 
             
            % imagetype = 0/1 for PM/AM object and arbitrary complex-valued -otherwise
            %[y_oF,A_ks,cond_A] = function_model_RRI_FDDT(zeros(Ny0,Nx0),uz,d,lambda,delta_y_0,beta,smoothing_back,imagetype);              
            [y_oF,xxx,A_ks] = FDDT(uz,2,d,lambda,delta_y_0,delta_x_0,q,smoothing_back,beta);
            [Nya,Nxa]=size(A_ks); clear A_ks
            if SpeedCntr
                p=toc;disp(['time cost for the RRI F-DDT algorithm = ' num2str(p) ' seconds'])                
            end
            
            pp=4; % <- add new position
            RMSE_opt_r(counter) = sqrt(mean(mean((abs(u0(1:Ny0_,1:Nx0_))-abs(y_oF(1:Ny0_,1:Nx0_))).^2)));
            
            xxx = angle(y_oF(1:Ny0_,1:Nx0_))-angle(u0(1:Ny0_,1:Nx0_)); mRRIFDDT = mean(xxx(:));
            RMSE_opt_ra(counter) = sqrt(mean(mean(( xxx-mRRIFDDT ).^2)));
        end
        %% the convolutional inverse using the transfer function with original...
        if illustrations(3) % conv 1 
            if SpeedCntr
                tic
            end
            
            mNy = max([Nyz Ny0]);mNx = max([Nxz Nx0]); % maxima of sizes of the object and sensor image
    % wavefield reconstruction with the convolutional inverse using the F-DDT
    % impulse matrix with original image size
            %u0FrS = function_conv(uz,d,lambda,mNy,mNx,delta_y_0);
            TF_conv1 = TransferFunctionFDDT(d,lambda,delta_y_0,delta_x_0,mNy,mNx,0);
            uz_ = zeros(mNy,mNx);uz_(mNy/2-Nyz/2+1:mNy/2+Nyz/2,mNx/2-Nxz/2+1:mNx/2+Nxz/2) = uz;
            u0FrS = ifft2(conj(TF_conv1).*fft2(uz_));
            u0FrS_=u0FrS(mNy/2-Ny0/2+1:mNy/2+Ny0/2,mNx/2-Nx0/2+1:mNx/2+Nx0/2);        
            
            if SpeedCntr
                p=toc;disp(['time cost for the conv-1 algorithm = ' num2str(p) ' seconds'])                
            end
            clear u0FrS
            qr=2;pp=4; % <- the table of images is rewritten as 2 rows, 4 columns
            RMSE_FreqS(counter) = sqrt(mean(mean((abs(u0(1:Ny0_,1:Nx0_)) - abs(u0FrS_(1:Ny0_,1:Nx0_))).^2)));
            
            xxx = angle(u0FrS_(1:Ny0_,1:Nx0_)) - angle(u0(1:Ny0_,1:Nx0_)); mconv1 = mean(xxx(:));
            RMSE_FreqSa(counter) = sqrt(mean(mean(( xxx-mconv1 ).^2)));            
        end
        %%
        if illustrations(4)  %  and double (conv2) image sizes   
            if SpeedCntr
                tic
            end
            
            Nxa = round((Nx0 + Nxz)/2)*2;  Nya = 2*(round(Ny0 + Nyz)/2);
            %u0FreqL = function_conv(uz,d,lambda,Nya,Nxa,delta_y_0); 
            TF_conv2 = TransferFunctionFDDT(d,lambda,delta_y_0,delta_x_0,Nya,Nxa,0);
            uz_ = zeros(Nya,Nxa);uz_(Nya/2-Nyz/2+1:Nya/2+Nyz/2,Nxa/2-Nxz/2+1:Nxa/2+Nxz/2) = uz;
            u0FreqL = ifft2(conj(TF_conv2).*fft2(uz_));
            u0FrL = u0FreqL(Nya/2-Ny0/2+1:Nya/2+Ny0/2,Nxa/2-Nx0/2+1:Nxa/2+Nx0/2);
            
            if SpeedCntr
                p=toc;disp(['time cost for the conv-2 algorithm = ' num2str(p) ' seconds'])                
            end
            clear u0FreqL
            qr=2;pp=4;
            RMSE_FreqL(counter) = sqrt(mean(mean((abs(u0(1:Ny0_,1:Nx0_)) - abs(u0FrL(1:Ny0_,1:Nx0_))).^2)));
            
            xxx = angle(u0FrL(1:Ny0_,1:Nx0_)) - angle(u0(1:Ny0_,1:Nx0_)); mconv2 = mean(xxx(:));
            RMSE_FreqLa(counter) = sqrt(mean(mean(( xxx - mconv2 ).^2)));
        end 
% ==================================== added 5.11 =========================
        if illustrations(5) %      ASD-1 
            if SpeedCntr
                tic
            end
            
            mNy = max([Nyz Ny0]);mNx = max([Nxz Nx0]); % maxima of sizes of the object and sensor image
            %TF_ASD1 = function_TransferFunction_ASD(d,lambda,delta_y_0,mNy,mNx);
            TF_ASD1 = TransferFunctionASD(d,lambda,delta_y_0,delta_x_0,mNy,mNx);
            uz_ = zeros(mNy,mNx);uz_(mNy/2-Nyz/2+1:mNy/2+Nyz/2,mNx/2-Nxz/2+1:mNx/2+Nxz/2) = uz;
            u0ASD = ifft2(conj(TF_ASD1).*fft2(uz_));
            u0ASD_1 = u0ASD(mNy/2-Ny0/2+1:mNy/2+Ny0/2,mNx/2-Nx0/2+1:mNx/2+Nx0/2);  
            
            if SpeedCntr
                p=toc;disp(['time cost for the ASD-1 algorithm = ' num2str(p) ' seconds'])                
            end
            
            clear u0ASD uz_
            qr=2;pp=4;   
            RMSE_ASD1(counter) = sqrt(mean(mean((abs(u0(1:Ny0_,1:Nx0_)) - abs(u0ASD_1(1:Ny0_,1:Nx0_))).^2)));
            
            xxx = angle(u0ASD_1(1:Ny0_,1:Nx0_)) - angle(u0(1:Ny0_,1:Nx0_)); mASD1 = mean(xxx(:));
            RMSE_ASD1a(counter) = sqrt(mean(mean(( xxx-mASD1 ).^2)));            
        end
        
        if illustrations(6) %   double size ASD-2 
            if SpeedCntr
                tic
            end
            
            Nxa = round((Nx0 + Nxz)/2)*2;  Nya = 2*(round(Ny0 + Nyz)/2);            
            %TF_ASD2 = function_TransferFunction_ASD(d,lambda,delta_y_0,Nya,Nxa);
            TF_ASD2 = TransferFunctionASD(d,lambda,delta_y_0,delta_x_0,Nya,Nxa);
            uz_ = zeros(Nya,Nxa);uz_(Nya/2-Nyz/2+1:Nya/2+Nyz/2,Nxa/2-Nxz/2+1:Nxa/2+Nxz/2) = uz;
            u0ASD = ifft2(conj(TF_ASD2).*fft2(uz_));
            u0ASD_2 = u0ASD(Nya/2-Ny0/2+1:Nya/2+Ny0/2,Nxa/2-Nx0/2+1:Nxa/2+Nx0/2);            
            
            if SpeedCntr
                p=toc;disp(['time cost for the ASD-2 algorithm = ' num2str(p) ' seconds'])                
            end
            
            clear u0ASD uz_
            qr=2;pp=4;
            RMSE_ASD2(counter) = sqrt(mean(mean((abs(u0(1:Ny0_,1:Nx0_)) - abs(u0ASD_2(1:Ny0_,1:Nx0_))).^2)));
            
            xxx = angle(u0ASD_2(1:Ny0_,1:Nx0_)) - angle(u0(1:Ny0_,1:Nx0_)); mASD2 = mean(xxx(:));
            RMSE_ASD2a(counter) = sqrt(mean(mean(( xxx - mASD2 ).^2)));
        end 
        clear xxx        
      end 
      waitbar((counter/length(setd)/length(bsetd)),h);
    end % distance backward
    
    if 1 % illustration (visual comparison of wavefield reconstruction with different methods)  
      if 1 % amplitude
        figure(2);
        subplot(qr,pp,1);imshow(abs(u0(1:Ny0_,1:Nx0_)),[]);title('Original image (Amplitude)')
        xlabel(['z=z_f x ' num2str(d/dF)])
        if 1 % M-DDT
            subplot(qr,pp,2);imshow(abs(u0est(1:Ny0_,1:Nx0_)),[]);title('M-DDT reconstruction')
            xlabel(['RMSE (MDDT) = ' num2str(RMSE_MDDT(counter))])
        end

        if illustrations(1)
            subplot(qr,pp,3);imshow(abs(u0DFrT(1:Ny0_,1:Nx0_)),[]);title('truncated SVD reconstruction')
            xlabel(['RMSE (trunc.SVD) = ' num2str(RMSE_iDFrT(counter))])
        end

        if illustrations(2) 
            subplot(qr,pp,4);imshow(abs(y_oF(1:Ny0_,1:Nx0_)),[]);title('F-DDT RRI reconstruction')
            xlabel(['RMSE (FDDT RRI) = ' num2str(RMSE_opt_r(counter))])
        end
        if illustrations(3) 
            subplot(qr,pp,5);imshow(abs(u0FrS_(1:Ny0_,1:Nx0_)),[]);title('single conv reconstruction')
            xlabel(['RMSE (conv1) = ' num2str(RMSE_FreqS(counter))])
        end
        if illustrations(4) 
            subplot(qr,pp,6);imshow(abs(u0FrL(1:Ny0_,1:Nx0_)),[]);title('double conv reconstruction')
            xlabel(['RMSE (conv2) = ' num2str(RMSE_FreqL(counter))])
        end
% -------------------------------- ASD-1,2 --------------------------------        
        if illustrations(5) 
            subplot(qr,pp,7);imshow(abs(u0ASD_1(1:Ny0_,1:Nx0_)),[]);title('single ASD reconstruction')
            xlabel(['RMSE (ASD1) = ' num2str(RMSE_ASD1(counter))])
        end
        if illustrations(6) 
            subplot(qr,pp,8);imshow(abs(u0ASD_2(1:Ny0_,1:Nx0_)),[]);title('double ASD reconstruction')
            xlabel(['RMSE (ASD2) = ' num2str(RMSE_ASD2(counter))])
        end
      end
      
      if  1   % phase ====================================================
        figure(3);
        subplot(qr,pp,1);imshow(angle(u0(1:Ny0_,1:Nx0_)),[]);title('Original image (Phase)')
        xlabel(['z=z_f x ' num2str(d/dF)])
        if 1
            subplot(qr,pp,2);imshow(angle(u0est)-mMDDT,[]);title('M-DDT reconstruction')
            xlabel(['RMSE (MDDT) = ' num2str(RMSE_MDDTa(counter))])
        end

        if illustrations(1)
            subplot(qr,pp,3);imshow(angle(u0DFrT(1:Ny0_,1:Nx0_))-mMiDFrT,[]);title('truncated SVD reconstruction')
            xlabel(['RMSE (trunc. SVD) = ' num2str(RMSE_iDFrTa(counter))])
        end

        if illustrations(2) 
            subplot(qr,pp,4);imshow(angle(y_oF(1:Ny0_,1:Nx0_))-mRRIFDDT,[]);title('F-DDT RRI reconstruction')
            xlabel(['RMSE (FDDT RRI) = ' num2str(RMSE_opt_ra(counter))])
        end
        if illustrations(3) 
            subplot(qr,pp,5);imshow(angle(u0FrS_(1:Ny0_,1:Nx0_))-mconv1,[]);title('single conv reconstruction')
            xlabel(['RMSE (conv1) = ' num2str(RMSE_FreqSa(counter))])
        end
        if illustrations(4) 
            subplot(qr,pp,6);imshow(angle(u0FrL(1:Ny0_,1:Nx0_))-mconv2,[]);title('double conv reconstruction')
            xlabel(['RMSE (conv2) = ' num2str(RMSE_FreqLa(counter))])
        end

 % -------------------------------- ASD-1,2 --------------------------------        
        if illustrations(5) 
            subplot(qr,pp,7);imshow(angle(u0ASD_1(1:Ny0_,1:Nx0_))-mASD1,[]);title('single ASD reconstruction')
            xlabel(['RMSE (ASD1) = ' num2str(RMSE_ASD1a(counter))])
        end
        if illustrations(6) 
            subplot(qr,pp,8);imshow(angle(u0ASD_2(1:Ny0_,1:Nx0_))-mASD2,[]);title('double ASD reconstruction')
            xlabel(['RMSE (ASD2) = ' num2str(RMSE_ASD2a(counter))])
        end       
        
      end
      
        if length(setd)>1 && d~=setd(end) && 0
            disp('press any key to continue')
            pause        
        end        
    end
    counter = counter+1;
  end
    close(h);
    % numerical comparison for all algorithms + a curve of rank of the matrix AyH*Ay as a function of distance  
    if counter>2 && 0
        figure(4);
        plot(setd,RMSE_MDDT,'b',setd,RMSE_opt_r,'r',setd,RMSE_FreqS,'m',setd,RMSE_FreqL,'--s',setd,RMSE_iDFrT,'-.',setd,RMSE_ASD1,'k--',setd,RMSE_ASD2,'k:');
        xlabel('d, m');ylabel('RMSE_{abs}');title('RMSE for the intensity reconstruction');
        grid on;legend('M-DDT','F-DDT','conv1','conv2','trunc.SVD','ASD1','ASD2')
        figure(5);
        plot(setd,RMSE_MDDTa,'b',setd,RMSE_opt_ra,'r',setd,RMSE_FreqSa,'m',setd,RMSE_FreqLa,'--s',setd,RMSE_iDFrTa,'-.',setd,RMSE_ASD1a,'k--',setd,RMSE_ASD2a,'k:');
        xlabel('d, m');ylabel('RMSE_{angle}');title('RMSE for the phase reconstruction');
        grid on;legend('M-DDT','F-DDT','conv1','conv2','trunc.SVD','ASD1','ASD2')
        figure(6);
        plot(setd,rankAy);
        xlabel('d, m');ylabel('rank(Ay_H*Ay)');
        grid on;    
    end    
%% ========================================================================

end